@@ -1,5 +1,5 @@
 /*
-  QtCurve (C) Craig Drummond, 2003 - 2007 Craig.Drummond@lycos.co.uk
+  QtCurve (C) Craig Drummond, 2003 - 2008 Craig.Drummond@lycos.co.uk
 
   ----
 
@@ -32,6 +32,7 @@
 #define QTC_MAX_FILENAME_LEN   1024
 #define QTC_MAX_INPUT_LINE_LEN 256
 #define QTC_FILE               "qtcurvestylerc"
+#define QTC_VERSION_KEY        "version"
 
 #ifdef CONFIG_READ
 static int c2h(char ch)
@@ -78,6 +79,10 @@ static EDefBtnIndicator toInd(const char *str, EDefBtnIndicator def)
             return IND_CORNER;
         if(0==memcmp(str, "colored", 7))
             return IND_COLORED;
+        if(0==memcmp(str, "tint", 4))
+            return IND_TINT;
+        if(0==memcmp(str, "glow", 4))
+            return IND_GLOW;
     }
 
     return def;
@@ -123,13 +128,15 @@ static EMouseOver toMouseOver(const char *str, EMouseOver def)
             return MO_COLORED;
         if(0==memcmp(str, "plastik", 7))
             return MO_PLASTIK;
+        if(0==memcmp(str, "glow", 4))
+            return MO_GLOW;
         if(0==memcmp(str, "false", 4) || 0==memcmp(str, "none", 4))
             return MO_NONE;
     }
     return def;
 }
 
-static EAppearance toAppearance(const char *str, EAppearance def)
+static EAppearance toAppearance(const char *str, EAppearance def, bool allowFade)
 {
     if(str)
     {
@@ -149,6 +156,16 @@ static EAppearance toAppearance(const char *str, EAppearance def)
             return APPEARANCE_INVERTED;
         if(0==memcmp(str, "bevelled", 8))
             return APPEARANCE_BEVELLED;
+        if(allowFade && 0==memcmp(str, "fade", 4))
+            return APPEARANCE_FADE;
+
+        if(0==memcmp(str, "customgradient", 14) && strlen(str)>14)
+        {
+            int i=atoi(&str[14]);
+
+            if(i>=1 && i<(QTC_NUM_CUSTOM_GRAD+1))
+                return (EAppearance)(APPEARANCE_CUSTOM1+(i-1));
+        }
     }
     return def;
 }
@@ -184,6 +201,8 @@ static ERound toRound(const char *str, ERound def)
             return ROUND_SLIGHT;
         if(0==memcmp(str, "full", 4))
             return ROUND_FULL;
+        if(0==memcmp(str, "extra", 5))
+            return ROUND_EXTRA;
     }
 
     return def;
@@ -223,7 +242,7 @@ static EEffect toEffect(const char *str, EEffect def)
     return def;
 }
 
-static EShading toShading(const char * str, EShading def)
+static EShading toShading(const char *str, EShading def)
 {
     if(str)
     {
@@ -238,7 +257,7 @@ static EShading toShading(const char * str, EShading def)
     return def;
 }
 
-static EStripe toStripe(const char * str, EStripe def)
+static EStripe toStripe(const char *str, EStripe def)
 {
     if(str)
     {
@@ -253,7 +272,7 @@ static EStripe toStripe(const char * str, EStripe def)
     return def;
 }
 
-static ESliderStyle toSlider(const char * str, ESliderStyle def)
+static ESliderStyle toSlider(const char *str, ESliderStyle def)
 {
     if(str)
     {
@@ -261,6 +280,10 @@ static ESliderStyle toSlider(const char * str, ESliderStyle def)
             return SLIDER_ROUND;
         if(0==memcmp(str, "plain", 5))
             return SLIDER_PLAIN;
+        if(0==memcmp(str, "r-round", 7))
+            return SLIDER_ROUND_ROTATED;
+        if(0==memcmp(str, "r-plain", 7))
+            return SLIDER_PLAIN_ROTATED;
         if(0==memcmp(str, "triangular", 10))
             return SLIDER_TRIANGULAR;
     }
@@ -268,6 +291,38 @@ static ESliderStyle toSlider(const char * str, ESliderStyle def)
     return def;
 }
 
+static EColor toEColor(const char *str, EColor def)
+{
+    if(str)
+    {
+        if(0==memcmp(str, "base", 4))
+            return ECOLOR_BASE;
+        if(0==memcmp(str, "dark", 4))
+            return ECOLOR_DARK;
+        if(0==memcmp(str, "background", 10))
+            return ECOLOR_BACKGROUND;
+    }
+
+    return def;
+}
+
+static EFocus toFocus(const char *str, EFocus def)
+{
+    if(str)
+    {
+        if(0==memcmp(str, "standard", 8))
+            return FOCUS_STANDARD;
+        if(0==memcmp(str, "highlight", 9))
+            return FOCUS_HIGHLIGHT;
+        if(0==memcmp(str, "background", 10))
+            return FOCUS_BACKGROUND;
+        if(0==memcmp(str, "filled", 6))
+            return FOCUS_FILLED;
+    }
+
+    return def;
+}
+
 #endif
 
 #ifdef CONFIG_WRITE
@@ -401,7 +456,7 @@ class QtCConfig
     QtCConfig(const QString &filename);
 
     bool            ok() const { return values.count()>0; }
-    const QString & readEntry(const char *key, const QString &def=QString::null);
+    const QString & readEntry(const QString &key, const QString &def=QString::null);
 
     private:
 
@@ -436,24 +491,33 @@ QtCConfig::QtCConfig(const QString &filename)
     }
 }
 
-inline const QString & QtCConfig::readEntry(const char *key, const QString &def)
+inline const QString & QtCConfig::readEntry(const QString &key, const QString &def)
 {
     return values.contains(key) ? values[key] : def;
 }
 
-inline QString readStringEntry(QtCConfig &cfg, const char *key)
+inline QString readStringEntry(QtCConfig &cfg, const QString &key)
 {
     return cfg.readEntry(key);
 }
 
-static int readNumEntry(QtCConfig &cfg, const char *key, int def)
+static int readNumEntry(QtCConfig &cfg, const QString &key, int def)
 {
     const QString &val(readStringEntry(cfg, key));
 
     return val.isEmpty() ? def : val.toInt();
 }
 
-static bool readBoolEntry(QtCConfig &cfg, const char *key, bool def)
+/*
+static double readDoubleEntry(QtCConfig &cfg, const QString &key, double def)
+{
+    const QString &val(readStringEntry(cfg, key));
+
+    return val.isEmpty() ? def : val.toDouble();
+}
+*/
+
+static bool readBoolEntry(QtCConfig &cfg, const QString &key, bool def)
 {
     const QString &val(readStringEntry(cfg, key));
 
@@ -545,6 +609,15 @@ static int readNumEntry(GHashTable *cfg, char *key, int def)
     return str ? atoi(str) : def;
 }
 
+/*
+static double readDoubleEntry(GHashTable *cfg, char *key, double def)
+{
+    char *str=readStringEntry(cfg, key);
+
+    return str ? atof(str) : def;
+}
+*/
+
 static gboolean readBoolEntry(GHashTable *cfg, char *key, gboolean def)
 {
     char *str=readStringEntry(cfg, key);
@@ -575,16 +648,22 @@ static gboolean readBoolEntry(GHashTable *cfg, char *key, gboolean def)
     opts->ENTRY=toRound(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);
 
 #define QTC_CFG_READ_DI(ENTRY) \
-    opts->ENTRY=((double)(readNumEntry(cfg, #ENTRY, ((int)(def->ENTRY*100))-100)+100))/100.0;
+    opts->ENTRY=((double)(readNumEntry(cfg, #ENTRY, (int)((def->ENTRY*100.0)-99.5))+100))/100.0;
 
+#define QTC_CFG_READ_DI_BOOL(ENTRY) \
+    if(readBoolEntry(cfg, #ENTRY, false)) \
+        opts->ENTRY=def->ENTRY; \
+    else \
+        opts->ENTRY=((double)(readNumEntry(cfg, #ENTRY, (int)((def->ENTRY*100.0)-99.5))+100))/100.0;
+    
 #define QTC_CFG_READ_TB_BORDER(ENTRY) \
     opts->ENTRY=toTBarBorder(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);
 
 #define QTC_CFG_READ_MOUSE_OVER(ENTRY) \
     opts->ENTRY=toMouseOver(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);
 
-#define QTC_CFG_READ_APPEARANCE(ENTRY, DEF) \
-    opts->ENTRY=toAppearance(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), DEF);
+#define QTC_CFG_READ_APPEARANCE(ENTRY, DEF, ALLOW_FADE) \
+    opts->ENTRY=toAppearance(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), DEF, ALLOW_FADE);
 
 /*
 #define QTC_CFG_READ_APPEARANCE(ENTRY) \
@@ -620,6 +699,28 @@ static gboolean readBoolEntry(GHashTable *cfg, char *key, gboolean def)
     ENTRY=toShading(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), DEF);
 #endif
 
+#define QTC_CFG_READ_ECOLOR(ENTRY) \
+    opts->ENTRY=toEColor(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);
+
+#define QTC_CFG_READ_FOCUS(ENTRY) \
+    opts->ENTRY=toFocus(QTC_LATIN1(readStringEntry(cfg, #ENTRY)), def->ENTRY);
+
+static void checkAppearance(EAppearance *ap, Options *opts)
+{
+    if(*ap>=APPEARANCE_CUSTOM1 && *ap<(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD))
+    {
+#ifdef __cplusplus
+        if(opts->customGradient.end()==opts->customGradient.find(*ap))
+#else
+        if(!opts->customGradient[*ap-APPEARANCE_CUSTOM1])
+#endif
+            if(ap==&opts->appearance)
+                *ap=APPEARANCE_FLAT;
+            else
+                *ap=opts->appearance;
+    }
+}
+
 #ifdef __cplusplus
 static bool readConfig(const QString &file, Options *opts, Options *def)
 #else
@@ -665,17 +766,22 @@ static bool readConfig(const char *file, Options *opts, Options *def)
 
         if(cfg)
         {
+#endif
+            int    i;
+#if 0
+            double version=readDoubleEntry(cfg, QTC_VERSION_KEY, 0.59); /* 0.59 was last version not to sore version number!*/
+TODO: New versions (>0.60) need to handle config changes...
 #endif
             QTC_CFG_READ_NUM(passwordChar)
             QTC_CFG_READ_ROUND(round)
             QTC_CFG_READ_DI(highlightFactor)
             QTC_CFG_READ_TB_BORDER(toolbarBorders)
-            QTC_CFG_READ_APPEARANCE(appearance, def->appearance)
+            QTC_CFG_READ_APPEARANCE(appearance, def->appearance, false)
             QTC_CFG_READ_BOOL(fixParentlessDialogs)
             QTC_CFG_READ_STRIPE(stripedProgress)
             QTC_CFG_READ_SLIDER(sliderStyle)
             QTC_CFG_READ_BOOL(animatedProgress)
-            QTC_CFG_READ_BOOL(lighterPopupMenuBgnd)
+            QTC_CFG_READ_DI_BOOL(lighterPopupMenuBgnd)
             QTC_CFG_READ_BOOL(embolden)
             QTC_CFG_READ_DEF_BTN(defBtnIndicator)
             QTC_CFG_READ_LINE(sliderThumbs)
@@ -685,14 +791,18 @@ static bool readConfig(const char *file, Options *opts, Options *def)
             QTC_CFG_READ_SHADE(shadeSliders, false)
             QTC_CFG_READ_SHADE(shadeMenubars, true)
             QTC_CFG_READ_SHADE(shadeCheckRadio, false)
-            QTC_CFG_READ_APPEARANCE(menubarAppearance, def->menubarAppearance)
-            QTC_CFG_READ_APPEARANCE(menuitemAppearance, opts->appearance)
-            QTC_CFG_READ_APPEARANCE(toolbarAppearance, def->toolbarAppearance)
+            QTC_CFG_READ_APPEARANCE(menubarAppearance, def->menubarAppearance, false)
+            QTC_CFG_READ_APPEARANCE(menuitemAppearance, opts->appearance, true)
+            QTC_CFG_READ_APPEARANCE(toolbarAppearance, def->toolbarAppearance, false)
+#if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
+            QTC_CFG_READ_APPEARANCE(selectionAppearance, def->selectionAppearance, false)
+#endif
             QTC_CFG_READ_LINE(toolbarSeparators)
             QTC_CFG_READ_LINE(splitters)
             QTC_CFG_READ_BOOL(customMenuTextColor)
             QTC_CFG_READ_MOUSE_OVER(coloredMouseOver)
             QTC_CFG_READ_BOOL(menubarMouseOver)
+            QTC_CFG_READ_BOOL(useHighlightForMenu)
             QTC_CFG_READ_BOOL(shadeMenubarOnlyWhenActive)
             QTC_CFG_READ_BOOL(thinnerMenuItems)
             QTC_CFG_READ_COLOR(customSlidersColor)
@@ -702,29 +812,42 @@ static bool readConfig(const char *file, Options *opts, Options *def)
             QTC_CFG_READ_COLOR(customCheckRadioColor)
             QTC_CFG_READ_SCROLLBAR(scrollbarType)
             QTC_CFG_READ_EFFECT(buttonEffect)
-            QTC_CFG_READ_APPEARANCE(lvAppearance, opts->appearance)
-            QTC_CFG_READ_APPEARANCE(tabAppearance, opts->appearance)
-            QTC_CFG_READ_APPEARANCE(sliderAppearance, opts->appearance)
-            QTC_CFG_READ_APPEARANCE(progressAppearance, opts->appearance)
-#ifndef QTC_PLAIN_FOCUS_ONLY
-            QTC_CFG_READ_BOOL(stdFocus)
-#endif
+            QTC_CFG_READ_APPEARANCE(lvAppearance, opts->appearance, false)
+            QTC_CFG_READ_APPEARANCE(tabAppearance, def->tabAppearance, false)
+            QTC_CFG_READ_APPEARANCE(activeTabAppearance, opts->tabAppearance, false)
+            QTC_CFG_READ_APPEARANCE(sliderAppearance, opts->appearance, false)
+            QTC_CFG_READ_APPEARANCE(progressAppearance, opts->appearance, false)
+            QTC_CFG_READ_APPEARANCE(progressGrooveAppearance, APPEARANCE_INVERTED, false)
+            QTC_CFG_READ_ECOLOR(progressGrooveColor)
+            QTC_CFG_READ_FOCUS(focus)
             QTC_CFG_READ_BOOL(lvLines)
             QTC_CFG_READ_BOOL(drawStatusBarFrames)
             QTC_CFG_READ_BOOL(fillSlider)
             QTC_CFG_READ_BOOL(roundMbTopOnly)
             QTC_CFG_READ_BOOL(borderMenuitems)
-            QTC_CFG_READ_BOOL(gradientPbGroove)
             QTC_CFG_READ_BOOL(darkerBorders)
             QTC_CFG_READ_BOOL(vArrows)
             QTC_CFG_READ_BOOL(xCheck)
             QTC_CFG_READ_BOOL(framelessGroupBoxes)
             QTC_CFG_READ_BOOL(inactiveHighlight)
             QTC_CFG_READ_BOOL(colorMenubarMouseOver)
+            QTC_CFG_READ_BOOL(crHighlight)
+            QTC_CFG_READ_BOOL(crButton)
+            QTC_CFG_READ_BOOL(fillProgress)
+            QTC_CFG_READ_BOOL(comboSplitter)
+            QTC_CFG_READ_BOOL(squareScrollViews)
+            QTC_CFG_READ_BOOL(highlightScrollViews)
+            QTC_CFG_READ_BOOL(sunkenScrollViews)
+            QTC_CFG_READ_BOOL(flatSbarButtons)
+#if defined __cplusplus || defined QTC_GTK2_MENU_STRIPE
+            QTC_CFG_READ_BOOL(menuStripe)
+            QTC_CFG_READ_APPEARANCE(menuStripeAppearance, def->menuStripeAppearance, false)
+#endif
+            QTC_CFG_READ_BOOL(gtkScrollViews)
 #ifdef __cplusplus
             QTC_CFG_READ_BOOL(stdSidebarButtons)
-            QTC_CFG_READ_BOOL(gtkScrollViews)
             QTC_CFG_READ_BOOL(gtkComboMenus)
+            QTC_CFG_READ_BOOL(colorTitlebarOnly)
 /*
 #else
             QTC_CFG_READ_BOOL(setDialogButtonOrder)
@@ -735,9 +858,253 @@ static bool readConfig(const char *file, Options *opts, Options *def)
 #endif
 #if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
             QTC_CFG_READ_BOOL(gtkButtonOrder)
+#endif
+#ifdef __cplusplus
+            QTC_CFG_READ_APPEARANCE(titlebarAppearance, opts->appearance, false)
+            QTC_CFG_READ_APPEARANCE(inactiveTitlebarAppearance, opts->titlebarAppearance, false)
+            QTC_CFG_READ_APPEARANCE(titlebarButtonAppearance, opts->titlebarAppearance, false)
+
+            if(APPEARANCE_BEVELLED==opts->titlebarAppearance)
+                opts->titlebarAppearance=APPEARANCE_GRADIENT;
+            else if(APPEARANCE_RAISED==opts->titlebarAppearance)
+                opts->titlebarAppearance=APPEARANCE_FLAT;
+
+            if(APPEARANCE_BEVELLED==opts->inactiveTitlebarAppearance)
+                opts->inactiveTitlebarAppearance=APPEARANCE_GRADIENT;
+            else if(APPEARANCE_RAISED==opts->inactiveTitlebarAppearance)
+                opts->inactiveTitlebarAppearance=APPEARANCE_FLAT;
 #endif
             QTC_CFG_READ_SHADING(shading, shading);
 
+#ifdef __cplusplus
+#if (defined QT_VERSION && (QT_VERSION >= 0x040000))
+            QStringList shades(readStringEntry(cfg, "customShades").split(',', QString::SkipEmptyParts));
+#else
+            QStringList shades(QStringList::split(',', readStringEntry(cfg, "customShades")));
+#endif
+
+            if(NUM_STD_SHADES==shades.size())
+            {
+                QStringList::ConstIterator it(shades.begin());
+                bool                       ok(true);
+
+                opts->customShades.resize(NUM_STD_SHADES);
+                for(i=0; i<NUM_STD_SHADES && ok; ++i, ++it)
+                    opts->customShades[i]=(*it).toDouble(&ok);
+
+                if(!ok)
+                    opts->customShades.clear();
+            }
+
+            for(i=APPEARANCE_CUSTOM1; i<(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD+1); ++i)
+            {
+                QString gradKey;
+
+                if(i==QTC_NUM_CUSTOM_GRAD)
+                    gradKey="sunkengradient";
+                else
+                    gradKey.sprintf("customgradient%d", (i-APPEARANCE_CUSTOM1)+1);
+
+#if (defined QT_VERSION && (QT_VERSION >= 0x040000))
+                QStringList vals(readStringEntry(cfg, gradKey).split(',', QString::SkipEmptyParts));
+#else
+                QStringList vals(QStringList::split(',', readStringEntry(cfg, gradKey)));
+#endif
+
+                if(vals.size())
+                    opts->customGradient.erase((EAppearance)i);
+
+                if(vals.size()>=5 && vals.size()%2) // Need odd number...
+                {
+                    QStringList::ConstIterator it(vals.begin()),
+                                               end(vals.end());
+                    bool                       ok(true);
+                    CustomGradient             grad;
+
+                    if((*it)=="true")
+                        grad.lightBorder=true;
+                    else if((*it)=="false")
+                        grad.lightBorder=false;
+                    else
+                        ok=false;
+
+                    if(ok)
+                    {
+                        int j;
+
+                        for(++it, j=0; it!=end && ok; ++it, ++j)
+                        {
+                            double pos=(*it).toDouble(&ok),
+                                   val=ok ? (*(++it)).toDouble(&ok) : 0.0;
+
+                            if(ok)
+                                ok=(pos>=0 && pos<=1.0) &&
+                                   (val>=0.0 && val<=2.0);
+
+                            if(ok)
+                                grad.grad.insert(Gradient(pos, val));
+                        }
+                    }
+
+                    if(ok)
+                        opts->customGradient[(EAppearance)i]=grad;
+                }
+            }
+#else
+            {
+            char *str=readStringEntry(cfg, "customShades");
+
+            if(str)
+            {
+                int j,
+                    comma=0;
+
+                for(j=0; str[j]; ++j)
+                    if(','==str[j])
+                        comma++;
+
+                if((NUM_STD_SHADES-1)==comma)
+                {
+                    bool ok=true;
+
+                    opts->customShades=malloc(sizeof(double)*NUM_STD_SHADES);
+
+                    for(j=0; j<comma+1 && str && ok; ++j)
+                    {
+                        char *c=strchr(str, ',');
+
+                        if(c || (str && NUM_STD_SHADES-1==comma))
+                        {
+                            if(c)
+                                *c='\0';
+                            opts->customShades[j]=atof(str);
+                            str=c+1;
+                        }
+                        else
+                            ok=false;
+                    }
+
+                    if(!ok)
+                    {
+                        free(opts->customShades);
+                        opts->customShades=0L;
+                    }
+                }
+            }
+            }
+
+            for(i=0; i<QTC_NUM_CUSTOM_GRAD+1; ++i)
+            {
+                char gradKey[18];
+                char *str;
+
+                if(i==QTC_NUM_CUSTOM_GRAD)
+                    sprintf(gradKey, "sunkengradient");
+                else
+                    sprintf(gradKey, "customgradient%d", i+1);
+                if((str=readStringEntry(cfg, gradKey)))
+                {
+                    int j,
+                        comma=0;
+
+                    for(j=0; str[j]; ++j)
+                        if(','==str[j])
+                            comma++;
+
+                    if(comma && opts->customGradient[i])
+                    {
+                        if(opts->customGradient[i]->grad)
+                            free(opts->customGradient[i]->grad);
+                        free(opts->customGradient[i]);
+                        opts->customGradient[i]=0L;
+                    }
+
+                    if(comma>=4 && 0==comma%2)
+                    {
+                        char *c=strchr(str, ',');
+
+                        if(c)
+                        {
+                            bool lb=false,
+                                 ok=true;
+
+                            *c='\0';
+                            if(0==memcmp(str, "true", 4))
+                                lb=true;
+                            else if(0==memcmp(str, "false", 4))
+                                lb=false;
+                            else
+                                ok=false;
+
+                            if(ok)
+                            {
+                                opts->customGradient[i]=malloc(sizeof(CustomGradient));
+                                opts->customGradient[i]->numGrad=comma/2;
+                                opts->customGradient[i]->grad=malloc(sizeof(Gradient) * opts->customGradient[i]->numGrad);
+                                opts->customGradient[i]->lightBorder=lb;
+
+                                str=c+1;
+                                for(j=0; j<comma && str && ok; j+=2)
+                                {
+                                    c=strchr(str, ',');
+
+                                    if(c)
+                                    {
+                                        *c='\0';
+                                        opts->customGradient[i]->grad[j/2].pos=atof(str);
+                                        str=c+1;
+                                        c=str ? strchr(str, ',') : 0L;
+
+                                        if(c || str)
+                                        {
+                                            if(c)
+                                                *c='\0';
+                                            opts->customGradient[i]->grad[j/2].val=atof(str);
+                                            str=c ? c+1 : c;
+                                        }
+                                        else
+                                            ok=false;
+                                    }
+                                    else
+                                        ok=false;
+                                }
+
+                                if(!ok)
+                                {
+                                    free(opts->customGradient[i]->grad);
+                                    free(opts->customGradient[i]);
+                                    opts->customGradient[i]=0L;
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+#endif
+
+            /* **Must** check appearance first, as the rest will default to this */
+            checkAppearance(&opts->appearance, opts);
+            checkAppearance(&opts->menubarAppearance, opts);
+            checkAppearance(&opts->menuitemAppearance, opts);
+            checkAppearance(&opts->toolbarAppearance, opts);
+            checkAppearance(&opts->lvAppearance, opts);
+            checkAppearance(&opts->tabAppearance, opts);
+            checkAppearance(&opts->activeTabAppearance, opts);
+            checkAppearance(&opts->sliderAppearance, opts);
+#ifdef __cplusplus
+            checkAppearance(&opts->titlebarAppearance, opts);
+            checkAppearance(&opts->inactiveTitlebarAppearance, opts);
+            checkAppearance(&opts->titlebarButtonAppearance, opts);
+#endif
+#if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
+            checkAppearance(&opts->selectionAppearance, opts);
+#endif
+#if defined __cplusplus || defined QTC_GTK2_MENU_STRIPE
+            checkAppearance(&opts->menuStripeAppearance, opts);
+#endif
+            checkAppearance(&opts->progressAppearance, opts);
+            checkAppearance(&opts->progressGrooveAppearance, opts);
+
 #ifndef __cplusplus
             releaseConfig(cfg);
 #endif
@@ -764,18 +1131,52 @@ static bool readConfig(const char *file, Options *opts, Options *def)
             if(APPEARANCE_BEVELLED==opts->tabAppearance)
                 opts->tabAppearance=APPEARANCE_GRADIENT;
 
+            if(APPEARANCE_BEVELLED==opts->activeTabAppearance)
+                opts->activeTabAppearance=APPEARANCE_GRADIENT;
+
+#if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
+            if(APPEARANCE_RAISED==opts->selectionAppearance)
+                opts->selectionAppearance=APPEARANCE_FLAT;
+            else if(APPEARANCE_BEVELLED==opts->selectionAppearance)
+                opts->selectionAppearance=APPEARANCE_GRADIENT;
+#endif
+#if defined __cplusplus || defined QTC_GTK2_MENU_STRIPE
+            if(APPEARANCE_RAISED==opts->menuStripeAppearance)
+                opts->menuStripeAppearance=APPEARANCE_FLAT;
+            else if(APPEARANCE_BEVELLED==opts->menuStripeAppearance)
+                opts->menuStripeAppearance=APPEARANCE_GRADIENT;
+#endif
             if(opts->highlightFactor<((100.0+MIN_HIGHLIGHT_FACTOR)/100.0) ||
                opts->highlightFactor>((100.0+MAX_HIGHLIGHT_FACTOR)/100.0))
                 opts->highlightFactor=DEFAULT_HIGHLIGHT_FACTOR;
 
+            if(opts->lighterPopupMenuBgnd>((100.0+MAX_LIGHTER_POPUP_MENU)/100.0))
+                opts->lighterPopupMenuBgnd=DEF_POPUPMENU_LIGHT_FACTOR;
+
             if(opts->animatedProgress && !opts->stripedProgress)
                 opts->animatedProgress=false;
 
-            if(opts->colorSelTab && APPEARANCE_GRADIENT!=opts->tabAppearance)
+            if(opts->colorSelTab && APPEARANCE_GRADIENT!=opts->activeTabAppearance &&
+                                    APPEARANCE_INVERTED!=opts->activeTabAppearance)
                 opts->colorSelTab=false;
 
+/*
+??
             if(SHADE_CUSTOM==opts->shadeMenubars || SHADE_BLEND_SELECTED==opts->shadeMenubars || !opts->borderMenuitems)
                 opts->colorMenubarMouseOver=true;
+*/
+
+            if(MO_GLOW==opts->coloredMouseOver && (EFFECT_NONE==opts->buttonEffect || opts->round<ROUND_FULL))
+                opts->coloredMouseOver=MO_COLORED;
+
+            if(IND_GLOW==opts->defBtnIndicator && (EFFECT_NONE==opts->buttonEffect || opts->round<ROUND_FULL))
+                opts->defBtnIndicator=IND_TINT;
+
+            if(opts->squareScrollViews || EFFECT_NONE==opts->buttonEffect)
+                opts->sunkenScrollViews=false;
+
+            if(opts->squareScrollViews)
+                opts->highlightScrollViews=false;
 
             return true;
         }
@@ -806,12 +1207,21 @@ static const char * getSystemConfigFile()
 static void defaultSettings(Options *opts)
 {
     /* Set hard-coded defaults... */
+#ifndef __cplusplus
+    int i;
+
+    for(i=0; i<QTC_NUM_CUSTOM_GRAD+1; ++i)
+        opts->customGradient[i]=0L;
+
+    opts->customShades=0L;
+#endif
+
     opts->contrast=7;
     opts->passwordChar=0x25CF;
     opts->highlightFactor=DEFAULT_HIGHLIGHT_FACTOR;
     opts->round=ROUND_FULL;
-    opts->lighterPopupMenuBgnd=true;
-    opts->animatedProgress=true;
+    opts->lighterPopupMenuBgnd=DEF_POPUPMENU_LIGHT_FACTOR;
+    opts->animatedProgress=false;
     opts->stripedProgress=STRIPE_DIAGONAL;
     opts->sliderStyle=SLIDER_TRIANGULAR;
     opts->highlightTab=true;
@@ -820,12 +1230,15 @@ static void defaultSettings(Options *opts)
     opts->appearance=APPEARANCE_DULL_GLASS;
     opts->lvAppearance=APPEARANCE_BEVELLED;
     opts->tabAppearance=APPEARANCE_GRADIENT;
+    opts->activeTabAppearance=APPEARANCE_GRADIENT;
     opts->sliderAppearance=APPEARANCE_DULL_GLASS;
     opts->menubarAppearance=APPEARANCE_GRADIENT;
     opts->menuitemAppearance=APPEARANCE_DULL_GLASS;
     opts->toolbarAppearance=APPEARANCE_GRADIENT;
     opts->progressAppearance=APPEARANCE_DULL_GLASS;
-    opts->defBtnIndicator=IND_COLORED;
+    opts->progressGrooveAppearance=APPEARANCE_INVERTED;
+    opts->progressGrooveColor=ECOLOR_BASE;
+    opts->defBtnIndicator=IND_TINT;
     opts->sliderThumbs=LINE_FLAT;
     opts->handles=LINE_DOTS;
     opts->shadeSliders=SHADE_BLEND_SELECTED;
@@ -838,32 +1251,48 @@ static void defaultSettings(Options *opts)
     opts->customMenuTextColor=false;
     opts->coloredMouseOver=MO_PLASTIK;
     opts->menubarMouseOver=true;
-    opts->shadeMenubarOnlyWhenActive=true;
+    opts->useHighlightForMenu=true;
+    opts->shadeMenubarOnlyWhenActive=false;
     opts->thinnerMenuItems=false;
     opts->scrollbarType=SCROLLBAR_KDE;
     opts->buttonEffect=EFFECT_NONE;
-#ifndef QTC_PLAIN_FOCUS_ONLY
-    opts->stdFocus=true;
-#endif
+    opts->focus=FOCUS_STANDARD;
     opts->lvLines=false;
     opts->drawStatusBarFrames=false;
     opts->fillSlider=true;
     opts->roundMbTopOnly=true;
-    opts->borderMenuitems=true;
-    opts->gradientPbGroove=true;
+    opts->borderMenuitems=false;
     opts->darkerBorders=false;
     opts->vArrows=false;
     opts->xCheck=false;
-    opts->framelessGroupBoxes=false;
-    opts->colorMenubarMouseOver=false;
+    opts->framelessGroupBoxes=true;
+    opts->colorMenubarMouseOver=true;
     opts->inactiveHighlight=false;
+    opts->crHighlight=false;
+    opts->crButton=false;
+    opts->fillProgress=true;
+    opts->comboSplitter=true;
+    opts->squareScrollViews=false;
+    opts->highlightScrollViews=false;
+    opts->sunkenScrollViews=false;
+    opts->flatSbarButtons=false;
+#if defined __cplusplus || defined QTC_GTK2_MENU_STRIPE
+    opts->menuStripe=false;
+    opts->menuStripeAppearance=APPEARANCE_GRADIENT;
+#endif
 #ifdef QTC_CONFIG_DIALOG
     opts->shading=SHADING_HSL;
 #endif
+
+#if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
+    opts->selectionAppearance=APPEARANCE_FLAT;
+#endif
+
 #ifdef __cplusplus
     opts->stdSidebarButtons=false;
     opts->gtkScrollViews=false;
     opts->gtkComboMenus=false;
+    opts->colorTitlebarOnly=false;
     opts->customMenubarsColor.setRgb(0, 0, 0);
     opts->customSlidersColor.setRgb(0, 0, 0);
     opts->customMenuNormTextColor.setRgb(0, 0, 0);
@@ -886,7 +1315,13 @@ static void defaultSettings(Options *opts)
 #if defined QTC_CONFIG_DIALOG || (defined QT_VERSION && (QT_VERSION >= 0x040000)) || !defined __cplusplus
     opts->gtkButtonOrder=false;
 #endif
-
+#ifndef __cplusplus
+#endif
+#ifdef __cplusplus
+    opts->titlebarAppearance=APPEARANCE_GRADIENT;
+    opts->inactiveTitlebarAppearance=APPEARANCE_GRADIENT;
+    opts->titlebarButtonAppearance=APPEARANCE_GRADIENT;
+#endif
     /* Read system config file... */
     {
     static const char * systemFilename=NULL;
@@ -897,6 +1332,11 @@ static void defaultSettings(Options *opts)
     if(systemFilename)
         readConfig(systemFilename, opts, opts);
     }
+
+#if !defined QTC_CONFIG_DIALOG && defined QT_VERSION && (QT_VERSION < 0x040000)
+    if(FOCUS_FILLED==opts->focus)
+        opts->focus=FOCUS_HIGHLIGHT;
+#endif
 }
 
 #endif
@@ -912,6 +1352,10 @@ static const char *toStr(EDefBtnIndicator ind)
             return "fontcolor";
         case IND_CORNER:
             return "corner";
+        case IND_TINT:
+            return "tint";
+        case IND_GLOW:
+            return "glow";
         default:
             return "colored";
     }
@@ -957,12 +1401,14 @@ static const char *toStr(EMouseOver mo)
             return "colored";
         case MO_NONE:
             return "none";
+        case MO_GLOW:
+            return "glow";
         default:
             return "plastik";
     }
 }
 
-static const char *toStr(EAppearance exp)
+static QString toStr(EAppearance exp)
 {
     switch(exp)
     {
@@ -980,8 +1426,17 @@ static const char *toStr(EAppearance exp)
             return "bevelled";
         case APPEARANCE_INVERTED:
             return "inverted";
-        default:
+        case APPEARANCE_SHINY_GLASS:
             return "shinyglass";
+        case APPEARANCE_FADE:
+            return "fade";
+        default:
+        {
+            QString app;
+
+            app.sprintf("customgradient%d", (exp-APPEARANCE_CUSTOM1)+1);
+            return app;
+        }
     }
 }
 
@@ -1010,6 +1465,8 @@ static const char *toStr(ERound exp)
             return "none";
         case ROUND_SLIGHT:
             return "slight";
+        case ROUND_EXTRA:
+            return "extra";
         default:
         case ROUND_FULL:
             return "full";
@@ -1094,13 +1551,48 @@ static const char *toStr(ESliderStyle s)
             return "plain";
         case SLIDER_TRIANGULAR:
             return "triangular";
+        case SLIDER_ROUND_ROTATED:
+            return "r-round";
+        case SLIDER_PLAIN_ROTATED:
+            return "r-plain";
         default:
         case SLIDER_ROUND:
             return "round";
     }
 }
 
+static const char *toStr(EColor s)
+{
+    switch(s)
+    {
+        case ECOLOR_BACKGROUND:
+            return "background";
+        case ECOLOR_DARK:
+            return "dark";
+        default:
+        case ECOLOR_BASE:
+            return "base";
+    }
+}
+
+static const char *toStr(EFocus f)
+{
+    switch(f)
+    {
+        default:
+        case FOCUS_STANDARD:
+            return "standard";
+        case FOCUS_HIGHLIGHT:
+            return "highlight";
+        case FOCUS_BACKGROUND:
+            return "background";
+        case FOCUS_FILLED:
+            return "filled";
+    }
+}
+
 #if QT_VERSION >= 0x040000
+#include <QTextStream>
 #define CFG config
 #else
 #define CFG (*cfg)
@@ -1131,7 +1623,7 @@ static const char *toStr(ESliderStyle s)
     if (!exportingStyle && def.ENTRY==opts.ENTRY) \
         CFG.deleteEntry(#ENTRY); \
     else \
-        CFG.writeEntry(#ENTRY, ((int)(opts.ENTRY*100))-100);
+        CFG.writeEntry(#ENTRY, (int)((opts.ENTRY*100.0)-99.5));
 
 #define CFG_WRITE_ENTRY_NUM(ENTRY) \
     if (!exportingStyle && def.ENTRY==opts.ENTRY) \
@@ -1152,7 +1644,7 @@ bool static writeConfig(KConfig *cfg, const Options &opts, const Options &def, b
             sprintf(filename, "%s/"QTC_FILE, xdg);
 
 #if QT_VERSION >= 0x040000
-            KConfig defCfg(filename, KConfig::OnlyLocal);
+            KConfig defCfg(filename, KConfig::SimpleConfig);
 #else
             KConfig defCfg(filename, false, false);
 #endif
@@ -1167,6 +1659,7 @@ bool static writeConfig(KConfig *cfg, const Options &opts, const Options &def, b
 #else
         cfg->setGroup(QTC_GROUP);
 #endif
+        CFG.writeEntry(QTC_VERSION_KEY, VERSION);
         CFG_WRITE_ENTRY_NUM(passwordChar)
         CFG_WRITE_ENTRY(round)
         CFG_WRITE_ENTRY_D(highlightFactor)
@@ -1176,7 +1669,7 @@ bool static writeConfig(KConfig *cfg, const Options &opts, const Options &def, b
         CFG_WRITE_ENTRY(stripedProgress)
         CFG_WRITE_ENTRY(sliderStyle)
         CFG_WRITE_ENTRY(animatedProgress)
-        CFG_WRITE_ENTRY(lighterPopupMenuBgnd)
+        CFG_WRITE_ENTRY_D(lighterPopupMenuBgnd)
         CFG_WRITE_ENTRY(embolden)
         CFG_WRITE_ENTRY(defBtnIndicator)
         CFG_WRITE_ENTRY_B(sliderThumbs, true)
@@ -1189,11 +1682,14 @@ bool static writeConfig(KConfig *cfg, const Options &opts, const Options &def, b
         CFG_WRITE_ENTRY_FORCE(menubarAppearance)
         CFG_WRITE_ENTRY_FORCE(menuitemAppearance)
         CFG_WRITE_ENTRY_FORCE(toolbarAppearance)
+        CFG_WRITE_ENTRY_FORCE(selectionAppearance)
+        CFG_WRITE_ENTRY_FORCE(menuStripeAppearance)
         CFG_WRITE_ENTRY_B(toolbarSeparators, true)
         CFG_WRITE_ENTRY_B(splitters, false)
         CFG_WRITE_ENTRY(customMenuTextColor)
         CFG_WRITE_ENTRY(coloredMouseOver)
         CFG_WRITE_ENTRY(menubarMouseOver)
+        CFG_WRITE_ENTRY(useHighlightForMenu)
         CFG_WRITE_ENTRY(shadeMenubarOnlyWhenActive)
         CFG_WRITE_ENTRY(thinnerMenuItems)
         CFG_WRITE_ENTRY(customSlidersColor)
@@ -1205,31 +1701,97 @@ bool static writeConfig(KConfig *cfg, const Options &opts, const Options &def, b
         CFG_WRITE_ENTRY(buttonEffect)
         CFG_WRITE_ENTRY_FORCE(lvAppearance)
         CFG_WRITE_ENTRY_FORCE(tabAppearance)
+        CFG_WRITE_ENTRY_FORCE(activeTabAppearance)
         CFG_WRITE_ENTRY_FORCE(sliderAppearance)
         CFG_WRITE_ENTRY_FORCE(progressAppearance)
-#ifndef QTC_PLAIN_FOCUS_ONLY
-        CFG_WRITE_ENTRY(stdFocus)
-#endif
+        CFG_WRITE_ENTRY_FORCE(progressGrooveAppearance)
+        CFG_WRITE_ENTRY(progressGrooveColor)
+        CFG_WRITE_ENTRY(focus)
         CFG_WRITE_ENTRY(lvLines)
         CFG_WRITE_ENTRY(drawStatusBarFrames)
         CFG_WRITE_ENTRY(fillSlider)
         CFG_WRITE_ENTRY(roundMbTopOnly)
         CFG_WRITE_ENTRY(borderMenuitems)
-        CFG_WRITE_ENTRY(gradientPbGroove)
         CFG_WRITE_ENTRY(darkerBorders)
         CFG_WRITE_ENTRY(vArrows)
         CFG_WRITE_ENTRY(xCheck)
         CFG_WRITE_ENTRY(framelessGroupBoxes)
         CFG_WRITE_ENTRY(inactiveHighlight)
-#ifdef __cplusplus
+        CFG_WRITE_ENTRY(crHighlight)
+        CFG_WRITE_ENTRY(crButton)
+        CFG_WRITE_ENTRY(fillProgress)
+        CFG_WRITE_ENTRY(comboSplitter)
+        CFG_WRITE_ENTRY(squareScrollViews)
+        CFG_WRITE_ENTRY(highlightScrollViews)
+        CFG_WRITE_ENTRY(sunkenScrollViews)
+        CFG_WRITE_ENTRY(flatSbarButtons)
+        CFG_WRITE_ENTRY(menuStripe)
         CFG_WRITE_ENTRY(stdSidebarButtons)
-#endif
+        CFG_WRITE_ENTRY_FORCE(titlebarAppearance)
+        CFG_WRITE_ENTRY_FORCE(inactiveTitlebarAppearance)
+        CFG_WRITE_ENTRY_FORCE(titlebarButtonAppearance)
 
         CFG_WRITE_ENTRY(gtkScrollViews)
         CFG_WRITE_ENTRY(gtkComboMenus)
+        CFG_WRITE_ENTRY(colorTitlebarOnly)
         CFG_WRITE_ENTRY(gtkButtonOrder)
         CFG_WRITE_ENTRY(mapKdeIcons)
         CFG_WRITE_ENTRY(shading)
+
+        for(int i=APPEARANCE_CUSTOM1; i<(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD+1); ++i)
+        {
+            CustomGradientCont::const_iterator cg(opts.customGradient.find((EAppearance)i));
+            QString                            gradKey;
+
+            if(i==(APPEARANCE_CUSTOM1+QTC_NUM_CUSTOM_GRAD))
+                gradKey="sunkengradient";
+            else
+                gradKey.sprintf("customgradient%d", (i-APPEARANCE_CUSTOM1)+1);
+
+            if(cg==opts.customGradient.end())
+                CFG.deleteEntry(gradKey);
+            else
+            {
+                QString     gradVal;
+#if QT_VERSION >= 0x040000
+                QTextStream str(&gradVal);
+#else
+                QTextStream str(&gradVal, IO_WriteOnly);
+#endif
+
+                str << (const char *)((*cg).second.lightBorder ? "true" : "false");
+
+                GradientCont::const_iterator it((*cg).second.grad.begin()),
+                                             end((*cg).second.grad.end());
+
+                for(; it!=end; ++it)
+                    str << ',' << (*it).pos << ',' << (*it).val;
+                CFG.writeEntry(gradKey, gradVal);
+            }
+        }
+
+        if(NUM_STD_SHADES==opts.customShades.size())
+        {
+            QString     shadeVal;
+#if QT_VERSION >= 0x040000
+            QTextStream str(&shadeVal);
+#else
+            QTextStream str(&shadeVal, IO_WriteOnly);
+#endif
+
+            ShadesCont::const_iterator it(opts.customShades.begin()),
+                                       end(opts.customShades.end());
+
+            for(int i=0; it!=end; ++it, ++i)
+                if(0==i)
+                    str << *it;
+                else
+                    str << ',' << *it;
+            CFG.writeEntry("customShades", shadeVal);
+        }
+        else
+            CFG.deleteEntry("customShades");
+
         cfg->sync();
         return true;
     }
